import { fetchCDNConfig, sdkConfigAPI } from './requests';
import { generateBucketedConfig } from '@devcycle/bucketing';
import { cache } from 'react';
import { DVCPopulatedUser } from '@devcycle/js-client-sdk';
import { ConfigSource } from '@devcycle/types';
const getPopulatedUser = cache((user, userAgent) => {
    return new DVCPopulatedUser(user, {}, undefined, undefined, userAgent !== null && userAgent !== void 0 ? userAgent : undefined);
});
// wrap this function in react cache to avoid redoing work for the same user and config
const generateBucketedConfigCached = cache(async (obfuscated, user, config, userAgent) => {
    var _a, _b;
    const populatedUser = getPopulatedUser(user, userAgent);
    // clientSDKKey is always defined for bootstrap config
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const clientSDKKey = config.clientSDKKey;
    if ((_a = config.debugUsers) === null || _a === void 0 ? void 0 : _a.includes((_b = user.user_id) !== null && _b !== void 0 ? _b : '')) {
        const bucketedConfigResponse = await sdkConfigAPI(clientSDKKey, obfuscated, populatedUser);
        return {
            bucketedConfig: {
                ...bucketedConfigResponse,
                clientSDKKey,
            },
        };
    }
    return {
        bucketedConfig: {
            ...generateBucketedConfig({ user: populatedUser, config }),
            clientSDKKey,
            sse: {
                url: config.sse
                    ? `${config.sse.hostname}${config.sse.path}`
                    : undefined,
                inactivityDelay: 1000 * 60 * 2,
            },
        },
    };
});
class CDNConfigSource extends ConfigSource {
    constructor(clientSDKKey) {
        super();
        this.clientSDKKey = clientSDKKey;
    }
    async getConfig(sdkKey, kind, obfuscated) {
        // this request will be cached by Next
        const cdnConfig = await fetchCDNConfig(sdkKey, this.clientSDKKey, obfuscated);
        if (!cdnConfig.ok) {
            const responseText = await cdnConfig.text();
            throw new Error('Could not fetch config: ' + responseText);
        }
        return {
            config: (await cdnConfig.json()),
            lastModified: cdnConfig.headers.get('last-modified'),
            metaData: {},
        };
    }
    // dummy implementation to make types happy, this method isn't used in Next
    getConfigURL() {
        return '';
    }
}
/**
 * Retrieve the config from CDN for the current request's SDK Key. This data will often be cached
 * Compute the bucketed config for the current request's user using that data, with local bucketing library
 * Cache the bucketed config for this request so that repeated calls to this function are memoized
 */
export const getBucketedConfig = async (sdkKey, clientSDKKey, user, options, userAgent) => {
    var _a;
    const cdnConfigSource = new CDNConfigSource(clientSDKKey);
    const configSource = (_a = options.configSource) !== null && _a !== void 0 ? _a : cdnConfigSource;
    const { config, lastModified } = await configSource.getConfig(sdkKey, 'bootstrap', !!options.enableObfuscation, '', true);
    const { bucketedConfig } = await generateBucketedConfigCached(!!options.enableObfuscation, user, config, userAgent);
    return {
        ...bucketedConfig,
        lastModified: lastModified !== null && lastModified !== void 0 ? lastModified : undefined,
    };
};
//# sourceMappingURL=bucketing.js.map